package ari.networktraffic;

import android.app.Activity;
import android.content.Context;
import android.util.Log;

import java.io.File;
import java.io.IOException;
import java.io.OutputStreamWriter;

/**
 * Logs messages to an internal file ...
 * Created by trachten on 8/26/15.
 */
public class FileLogger extends LogHandler {
    private File recordFile; // The file to which to record data.
    private OutputStreamWriter osw; // Output streamer
    private Activity myActivity;    // needed to connect to the package manager

    /**
     * Register a new file logger to file named [fileName]
     * @param fileName The name of the internal file in which to store the results.
     *
     */
    public FileLogger(String fileName, Activity ActivityContext) {
        super();
        myActivity = ActivityContext;
        Log.i("NoPerms", myActivity.getFilesDir().toString());
        recordFile = new File(myActivity.getFilesDir() + "/" +fileName);
        try {
            osw = new OutputStreamWriter(myActivity.openFileOutput(fileName, Context.MODE_PRIVATE));
        } catch (IOException e) {
            Log.e("ERROR", "Could not open logging file for OUTPUT: " + e);
        }
    }

    @Override
    protected void store(String datum) {
        try {
            osw.write(datum);
            osw.flush();
        } catch (IOException e) {
            Log.e("ERROR", "Could not write logging data");
        }
    }

    protected void finalize() throws Throwable {
        try { // thanks http://stackoverflow.com/questions/2506488/when-is-the-finalize-method-called-in-java
            osw.close();
        } finally { super.finalize(); }
    }
}
