package ari.networktraffic;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.net.TrafficStats;
import android.net.Uri;
import android.telephony.TelephonyManager;
import android.util.Pair;

import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.List;

/**
 * Collections and logs data from the phone
 * Created by trachten on 7/16/15.
 */

public class GetData implements Runnable {
    private String[] alexa100={"google.com", "facebook.com", "youtube.com", "baidu.com", "yahoo.com", "amazon.com", "wikipedia.org", "qq.com", "google.co.in", "twitter.com", "taobao.com", "live.com", "sina.com.cn", "yahoo.co.jp", "weibo.com", "msn.com", "google.co.jp", "vk.com", "bing.com", "yandex.ru", "hao123.com", "linkedin.com", "ebay.com", "amazon.co.jp", "instagram.com", "google.de", "mail.ru", "netflix.com", "google.co.uk", "t.co", "pinterest.com", "reddit.com", "tmall.com", "360.cn", "google.ru", "google.com.br", "sohu.com", "paypal.com", "google.fr", "microsoft.com", "tumblr.com", "wordpress.com", "imdb.com", "blogspot.com", "imgur.com", "gmw.cn", "google.it", "onclickads.net", "aliexpress.com", "google.es", "apple.com", "xvideos.com", "ok.ru", "ask.com", "amazon.de", "fc2.com", "google.com.hk", "tianya.cn", "diply.com", "google.com.mx", "bongacams.com", "alibaba.com", "google.ca", "rakuten.co.jp", "stackoverflow.com", "google.com.tr", "amazon.co.uk", "pornhub.com", "soso.com", "go.com", "naver.com", "blogger.com", "xinhuanet.com", "craigslist.org", "kat.cr", "amazon.in", "google.co.id", "nicovideo.jp", "outbrain.com", "chinadaily.com.cn", "office.com", "pixnet.net", "xhamster.com", "github.com", "haosou.com", "google.pl", "google.com.au", "wikia.com", "cntv.cn", "coccoc.com", "cnn.com", "flipkart.com", "adnetworkperformance.com", "googleusercontent.com", "adobe.com", "bbc.co.uk", "sogou.com", "espn.go.com", "jd.com", "china.com"};
    public static boolean allStop=false; // stops further processing when pressed

    private Activity myActivity;
    /**
     * The Activity that is being run
     */
    private LogHandler myLogger;

    /**
     * The logging agent used to record results.
     */

    public GetData(Activity theActivity, LogHandler theLogger) {
        myActivity = theActivity;
        myLogger = theLogger;
    }

    /**
     * Record the fields of an object using refleciton.
     *
     * @param obj The object whose fields are to be recorded.
     */
    public void recordFields(Object obj) {
        for (Field field : obj.getClass().getDeclaredFields()) {
            Object value = null;
            try {
                value = field.get(obj);
            } catch (IllegalAccessException e) {
            }
            if (value != null)
                if (value instanceof Object[])
                    myLogger.record(field.getName() + ": " + Arrays.deepToString((Object[]) value));
                else if (value instanceof int[])
                    myLogger.record(field.getName() + ": " + Arrays.toString((int[]) value));
                else
                    myLogger.record(field.getName() + ": " + value);
        }
    }


    public void run() {
        // TELEPHONY
        myLogger.setPrefix("Telephony:  ");
        TelephonyManager tm = (TelephonyManager) myActivity.getSystemService(Context.TELEPHONY_SERVICE);
        myLogger.record("Call state: " + tm.getCallState());
        myLogger.record("Data state: " + tm.getDataState());
        myLogger.record("Data activity state:  " + tm.getDataActivity());
        myLogger.record("MMS user agent:  " + tm.getMmsUserAgent() + " and profile URL: " + tm.getMmsUAProfUrl());
        String phoneType = "";
        switch (tm.getPhoneType()) {
            case TelephonyManager.PHONE_TYPE_NONE:
                phoneType = "NONE";
                break;
            case TelephonyManager.PHONE_TYPE_CDMA:
                phoneType = "CDMA";
                break;
            case TelephonyManager.PHONE_TYPE_GSM:
                phoneType = "GSM";
                break;
            case TelephonyManager.PHONE_TYPE_SIP:
                phoneType = "SIP";
                break;
        }
        myLogger.record("Phone type: " + phoneType);
        myLogger.setPrefix("Telephony:  Network:  ");
        myLogger.record("Operator Country code:  " + tm.getNetworkCountryIso());
        myLogger.record("Operator MCC+MNC:  " + tm.getNetworkOperator());
        myLogger.record("Operator Name: " + tm.getNetworkOperatorName());
        myLogger.record("Roaming: " + tm.isNetworkRoaming());

        myLogger.setPrefix("Telephony:  SIM:  ");
        myLogger.record("Country code:  " + tm.getSimCountryIso());
        myLogger.record("Operator MCC+MNC:  " + tm.getSimOperator());
        myLogger.record("Name: " + tm.getSimOperatorName());
        myLogger.record("State: " + tm.getSimState());

        // TRAFFIC STATS
        myLogger.setPrefix("Traffic Stats:  ");
        TrafficStats ts = new TrafficStats();
        myLogger.record("Mobile received bytes: " + ts.getMobileRxBytes());
        myLogger.record("Mobile received packets: " + ts.getMobileRxPackets());
        myLogger.record("Mobile transmitted bytes: " + ts.getMobileTxBytes());
        myLogger.record("Mobile transmitted packets: " + ts.getMobileTxPackets());
        myLogger.record("Total received bytes: " + ts.getTotalRxBytes());
        myLogger.record("Total received packets: " + ts.getTotalRxPackets());
        myLogger.record("Total transmitted bytes: " + ts.getTotalRxBytes());
        myLogger.record("Total transmitted packets: " + ts.getTotalTxPackets());

        int chromeID = 0; // the ID of the skype app

        PackageManager pManager = myActivity.getPackageManager();
        List<ApplicationInfo> appList = pManager.getInstalledApplications(PackageManager.GET_ACTIVITIES |
                PackageManager.GET_META_DATA | PackageManager.GET_SHARED_LIBRARY_FILES | PackageManager.GET_UNINSTALLED_PACKAGES);

        if (appList == null)
            myLogger.record("no installed applications");
        else
            for (ApplicationInfo packageInfo : appList) {
                // get more information
                try {
                    ApplicationInfo aInfo = pManager.getApplicationInfo(packageInfo.packageName,
                            PackageManager.GET_META_DATA | PackageManager.GET_SHARED_LIBRARY_FILES | PackageManager.GET_UNINSTALLED_PACKAGES);
                    if (packageInfo.packageName.contains("com.android.chrome")) { // if this is chrome, record it's uid
                        chromeID = aInfo.uid;
                        myLogger.record("Chrome id is " + chromeID);
                    }


                } // ignore exceptions
                catch (PackageManager.NameNotFoundException | NullPointerException e) {
                }
            }

        // loop through the various sites
        for (String site : alexa100) {
            if (allStop) {
                return;
            }

            myLogger.setPrefix(site+":  ");

            // to time granularity measurements
            // ... make finalized copies of variables for granularity test
            final int myChromeID = chromeID;
            final TrafficStats myTs=ts;
            recordNetwork rn = new recordNetwork(myLogger,
                    "Chrome [uid " + chromeID + "]: ",
                    new recordNetwork.Function() {
                        @Override
                        public Pair<Long,Long> calc() {
                            return new Pair<Long,Long>(
                                    myTs.getUidTxBytes(myChromeID),
                                    myTs.getUidTxBytes(myChromeID)
                            );
                        }
                    }
            );
            Thread dataThread = new Thread(rn);
            dataThread.start(); // start logging network usage

            // ... open up some traffic-intensive URL
            Intent in = new Intent(Intent.ACTION_VIEW);
            in.setData(Uri.parse("http://"+site));
            myActivity.startActivity(in); // open web page, in order generate some traffic*/

            try {
                dataThread.join();
            } catch (InterruptedException e) {
                myLogger.record("Action interrupted!");
            }
        }

        myLogger.record("all done!");
    }
}
