package ari.networktraffic;

import java.util.Vector;

/**
 * Overarching interface for all logging functionality for this app
 * Created by trachten on 7/16/15.
 */
public abstract class LogHandler {

    // Priority constants - lower numbers subsume higher numbers
     static int VERBOSE = 0;       /** All log messages. */
     static int DEBUGGING = 1;     /** Debugging messages. */
     static int FAILURES = 2;      /** Log all failures. */
     static int INFO = 4;          /** General info messages. */
     static int SUCCESSES = 6;     /** Only log successful invokations. */

    // constants
    final static int DEFAULT_PRIORITY = DEBUGGING; /* The default priority to use for log messages. */

    LogHandler() {
        myPrefix=new Vector<String>();
    }
    /**
     * Records a string [datum] in the log with the given priority
     * @param priority The priority level of the record
     * @param datum The datum to record
     */
     void record(int priority, String datum) {
         if (priority>=myPriority) {
             // generate the prefix
             String prefix="";
             for (int ii = 0; ii < myPrefix.size(); ii++)
                 if (myPrefix.get(ii)!=null)
                     prefix+=myPrefix.get(ii);
                 else
                     prefix+="   ";
             store(prefix + datum);
         }
     }

    /**
     * Records a string [datum] in the log with the default priority
     * @param datum The datum to record
     */
    void record(String datum) {record(VERBOSE,datum);}

    /**
     * Stores the datum to the logger
     * @param datum The item to store
     */
    abstract protected void store(String datum);

    /**
     * Sets a prefix for future message at a given level, resetting all prefixes at higher levels
     * at the same time.  When a message is recorded, all known prefixes are first printed out.
     * @param level The level at which to add the prefix.
     * @param thePrefix The prefix to add.
     */
    public void setPrefix(int level, String thePrefix) {
        myPrefix.setSize(level+1);
        myPrefix.set(level,thePrefix);
    }

    /**
     * Set prefix at the default level 0.
     */
    public void setPrefix(String thePrefix) {
        setPrefix(0,thePrefix);
    }

    private String myIdent;     /** An identifier associated with this log. */
    private Vector<String> myPrefix; /** A prefix to add to all logs. */
    private int myPriority;     /** The desired priority to log. */

}
